@@ -1,4 +1,5 @@
 import inspect
+import logging
 from importlib import import_module
 from typing import Any, List, Optional, Protocol, Union
 from warnings import warn
@@ -17,6 +18,9 @@ class _CausalmlEstimator(Protocol):
         ...
 
 
+logger = logging.getLogger(__name__)
+
+
 class Causalml(CausalEstimator):
     """Wrapper class for estimators from the causalml library.
 
@@ -88,8 +92,12 @@ class Causalml(CausalEstimator):
                 DeprecationWarning,
                 stacklevel=2,
             )
-            estimator_class = self._get_causalml_class_object(causalml_estimator)
-            self.estimator = estimator_class(**kwargs["init_params"])
+            try:
+                estimator_class = self._get_causalml_class_object(causalml_estimator)
+                self.estimator = estimator_class(**kwargs["init_params"])
+            except ImportError:
+                logger.error("You must install https://github.com/uber/causalml to use this functionality")
+                raise
         else:
             self.estimator = causalml_estimator
         self.logger.info("INFO: Using CausalML Estimator")
