@@ -1,7 +1,7 @@
 import os
 import tempfile
 from copy import deepcopy
-from typing import Any, Dict, Optional, Tuple
+from typing import Any, Dict, Optional, Tuple, Union
 
 import networkx as nx
 import numpy as np
@@ -13,6 +13,7 @@ def _plot_causal_graph_graphviz(
     causal_graph: nx.Graph,
     display_causal_strengths: bool = True,
     causal_strengths: Optional[Dict[Tuple[Any, Any], float]] = None,
+    colors: Optional[Dict[Union[Any, Tuple[Any, Any]], str]] = None,
     filename: Optional[str] = None,
     display_plot: bool = True,
     figure_size: Optional[Tuple[int, int]] = None,
@@ -21,6 +22,10 @@ def _plot_causal_graph_graphviz(
         causal_strengths = {}
     else:
         causal_strengths = deepcopy(causal_strengths)
+    if colors is None:
+        colors = {}
+    else:
+        colors = deepcopy(colors)
 
     max_strength = 0.0
     for (source, target, strength) in causal_graph.edges(data="CAUSAL_STRENGTH", default=None):
@@ -28,14 +33,20 @@ def _plot_causal_graph_graphviz(
             causal_strengths[(source, target)] = strength
         if causal_strengths[(source, target)] is not None:
             max_strength = max(max_strength, abs(causal_strengths[(source, target)]))
+        if (source, target) not in colors:
+            colors[(source, target)] = "black"
 
     pygraphviz_graph = pygraphviz.AGraph(directed=isinstance(causal_graph, nx.DiGraph))
 
     for node in causal_graph.nodes:
-        pygraphviz_graph.add_node(node)
+        if node in colors:
+            pygraphviz_graph.add_node(node, color=colors[node], fontcolor=colors[node])
+        else:
+            pygraphviz_graph.add_node(node)
 
     for (source, target) in causal_graph.edges():
         causal_strength = causal_strengths[(source, target)]
+        color = colors[(source, target)]
         if causal_strength is not None:
             if np.isinf(causal_strength):
                 causal_strength = 10000
@@ -48,9 +59,10 @@ def _plot_causal_graph_graphviz(
                 str(target),
                 label=tmp_label if display_causal_strengths else None,
                 penwidth=str(_calc_arrow_width(causal_strength, max_strength)),
+                color=color,
             )
         else:
-            pygraphviz_graph.add_edge(str(source), str(target))
+            pygraphviz_graph.add_edge(str(source), str(target), color=color)
 
     pygraphviz_graph.layout(prog="dot")
     if filename is not None:
