@@ -29,3 +29,11 @@ def test_given_causal_strengths_when_plot_graph_then_does_not_modify_input_objec
     plot(nx.DiGraph([("X", "Y"), ("Y", "Z")]), causal_strengths=causal_strength)
 
     assert causal_strength == {("X", "Y"): 10}
+
+
+def test_given_colors_when_plot_graph_then_does_not_modify_input_object():
+    colors = {("X", "Y"): "red", "X": "blue"}
+
+    plot(nx.DiGraph([("X", "Y"), ("Y", "Z")]), colors=colors)
+
+    assert colors == {("X", "Y"): "red", "X": "blue"}
